/*
 * AllAlgorithms.h
 *
 *  Created on: 8 avr. 2010
 *      Author: jnschaeffer
 *
 * Copyright 2010 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * Includes all of the search algorithms available, plain and simple.
 */
#ifndef ALLALGORITHMS_H_
#define ALLALGORITHMS_H_

#include "../search/AlgorithmRegistration.h"

#include "RandomMonteCarlo.h"
#include "MinimaxMonteCarlo.h"
#include "MinimaxUCT.h"
#include "UCT.h"
#include "RandomSearchAlg.h"
#include "KFailure.h"
#include "MRun.h"
#include "MLearning.h"
#include "Minimax.h"
#include "UCTV.h"
#include "UCTVInf.h"
#include "UCTTuned.h"

BEGIN_REGISTER_ALGS;

REGISTER_ALG_3(uct, UCT);
REGISTER_ALG_2(random_montecarlo,	RandomMonteCarlo);
REGISTER_ALG_0(random,				RandomSearchAlg);
REGISTER_ALG_0(minimax,				Minimax);
REGISTER_ALG_2(minimax_montecarlo,	MinimaxMonteCarlo);
REGISTER_ALG_3(minimax_uct,			MinimaxUCT);
REGISTER_ALG_2(failure,				KFailure);
REGISTER_ALG_N(run,					MRun, 2, args[0], false, args[1]);
REGISTER_ALG_N(run_nonrecalling,	MRun, 2, args[0], true, args[1]);
REGISTER_ALG_2(learning,			MLearning);
REGISTER_ALG_3(uctv,				UCTV);
REGISTER_ALG_N(uctvinf,				UCTVInf, 5, args[0], args[1], args[2], args[3], args[4], false);
REGISTER_ALG_N(uctvinf_air,			UCTVInf, 5, args[0], args[1], args[2], args[3], args[4], true);
REGISTER_ALG_3(ucttuned,			UCTTuned);

END_REGISTER_ALGS;

#endif /* ALLALGORITHMS_H_ */
