/*
 * MLearning.cpp
 *
 *  Created on: Apr 1, 2010
 *      Author: dbjorge
 *
 * Copyright 2010 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "MLearning.h"

MLearning::MLearning(unsigned int m, double effectiveness=0.5) :
	RolloutMonteCarlo(effectiveness, 0.0),
	mM(m),
	mRound(0),
	mPlayForever(false),
	mNextAction(0)
{ }

MLearning::~MLearning()
{ }

void MLearning::reset() {
	mRound = 0;
	mPlayForever = false;
	mNextAction = 0;
}

void MLearning::iterate() {
	if(mPlayForever) return; //Not changing

	GameState& state = mTree->getRoot();

	GameState& chosenChild = state.getChild(mNextAction);
	player_t simulatedWinner = simulate(chosenChild);

	if(simulatedWinner == A) {
		chosenChild.searchData.customUInt++;
	} else {
		mNextAction = (mNextAction + 1) % state.countChildren();
	}

	mRound++;

	if(mRound >= mM) {
		mPlayForever = true;

		unsigned int bestHistory = 0;
		action_t bestAction = 0;
		for(int i=0;i<state.countChildren();i++) {
			SearchData data = state.getChildSearchData(i);
			if(data.customUInt > bestHistory) {
				bestHistory = data.customUInt;
				bestAction = i;
			}
		}

		mNextAction = bestAction;
	}
}

action_t MLearning::currentBestMove() {
	return mNextAction;
}

std::string MLearning::toString() const {
	std::stringstream out;
	out << mM << "-learning";
	return out.str();
}
