/*
 * MRun.cpp
 *
 *  Created on: Apr 1, 2010
 *      Author: dbjorge
 *
 * Copyright 2010 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "MRun.h"
MRun::MRun(unsigned int m, bool recalling=true, double effectiveness=0.5) :
	RolloutMonteCarlo(effectiveness, 0.0),
	mM(m),
	mRecalling(recalling),
	mPlayForever(false),
	mNextAction(0),
	mCurrentHistory(0),
	mArmsPlayed(0),
	mBestArm(0),
	mBestHistory(0)
{ }

MRun::~MRun()
{ }

void MRun::reset() {
	mPlayForever = false;
	mNextAction = 0;
	mCurrentHistory = 0;
	mArmsPlayed = 0;
	mBestArm = 0;
	mBestHistory = 0;
}

void MRun::iterate() {
	if(mPlayForever) return; //Not changing

	GameState& state = mTree->getRoot();

	GameState& chosenChild = state.getChild(mNextAction);
	player_t simulatedWinner = simulate(chosenChild);

	if(simulatedWinner == A) {
		mCurrentHistory++;
		if(mCurrentHistory >= mM) {
			mPlayForever = true;
		}
	} else {
		if(mCurrentHistory > mBestHistory) {
			mBestHistory = mCurrentHistory;
			mBestArm = mNextAction;
		}

		mCurrentHistory = 0;
		mNextAction = (mNextAction + 1) % state.countChildren();
		mArmsPlayed++;

		if(mArmsPlayed >= mM && mRecalling) {
			mPlayForever = true;
			mNextAction = mBestArm;
		}
	}
}

action_t MRun::currentBestMove() {
	return mBestArm;
}

std::string MRun::toString() const {
	std::stringstream out;
	out << mM << "-run";
	if(!mRecalling) out << "-nonrecalling";
	return out.str();
}

