/*
 * Minimax.cpp
 *
 *  Created on: Apr 1, 2010
 *      Author: dbjorge
 *
 * Copyright 2010 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "Minimax.h"

Minimax::Minimax() :
	mFinished(false),
	mChosenAction(0)
{ }

Minimax::~Minimax()
{ }

std::string Minimax::toString() const {
	return "minimax";
}

void Minimax::reset() {
	mFinished = false;
}

void Minimax::iterate() {
	if(mFinished) return;

	GameState& root = mTree->getRoot();
	evaluate(root);

	//Set mChosenAction to an optimal action.
	double minScore = 1.0;
	double score;
	for(action_t i=0;i<root.countChildren();i++) {
		score = root.getChild(i).searchData.customDouble;
		if(score < minScore) {
			minScore = score;
			mChosenAction = i;
		}
	}
	mFinished = true;
}

action_t Minimax::currentBestMove() {
	return mChosenAction;
}

double Minimax::evaluate(GameState& state) {
	bool isMinimizing = state.getPlayer() == A;
	double value = isMinimizing ? 1.0 : 0.0;

	if(state.isTerminal())  {
		value = (state.getWinner() == A) ? 0.0 : 1.0;
	} else {
		double childValue;
		for(int i=0;i<state.countChildren();i++) {
			childValue = evaluate(state.getChild(i));
			if((isMinimizing && childValue < value) || (!isMinimizing && childValue > value)) {
				value = childValue;
			}
		}
	}

	state.searchData.customDouble = value;
	return value;
}
