/*
 * UCT.cpp
 *
 *  Created on: 2010.03.24.
 *      Author: jnschaeffer
 *
 * Copyright 2010 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "UCT.h"
#include <math.h>
#include <time.h>
#include <cfloat>

UCT::UCT(double effectiveness, double discount, double fpuConst):
	RolloutMonteCarlo(effectiveness, discount),
	explorationConst(0.7),
	fpuConst(fpuConst)
{ }

UCT::~UCT()
{ }

double UCT::evaluate(GameState& state) {
	double bias = explorationBias(state);
	double value;

	if(state.getPlayer() == B) { //we like low values
		value = 1 - state.searchData.empiricalValue;
	} else { //we like high values
		value = state.searchData.empiricalValue;
	}

	return value + bias;
}

double UCT::evaluateFirst(GameState& state) {
	return fpuConst;
}

double UCT::explorationBias(GameState& state) {
	double logParentVisits = log(state.getParent()->searchData.numVisits);

	double bias = explorationConst * sqrt(logParentVisits/state.searchData.numVisits);

	return bias;
}

std::string UCT::toString() const {
	std::stringstream ss;
	ss << "uct";
	if(fpuConst <= 9999) ss << "-fpu-" << fpuConst;
	return ss.str();
}
