/*
 * UCTV.cpp
 *
 *  Created on: 7 avr. 2010
 *      Author: jnschaeffer
 *
 * Copyright 2010 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "UCTV.h"
#include <math.h>

UCTV::UCTV(double effectiveness, double propogation, double fpu):
	UCT(effectiveness,propogation, fpu)
{ }

double UCTV::explorationBias(GameState& state) {

	double nonDecSeq = log(state.getParent()->searchData.numVisits+1);

	double firstTerm = 2*state.searchData.estimatedValue*nonDecSeq;
	firstTerm /= state.searchData.numVisits+1;
	firstTerm = sqrt(firstTerm);

	return firstTerm + (3*nonDecSeq)/(state.searchData.numVisits+1);

}

void UCTV::update(GameState& state, double value) {
	SearchData& data = state.searchData;

	// E[X^2]
	data.customDouble =
				(data.customDouble * (data.numVisits) + value*value) /
				(data.numVisits+1);

	UCT::update(state, value);

	data.estimatedValue = data.customDouble - data.empiricalValue*data.empiricalValue;

}

UCTV::~UCTV() {
}

std::string UCTV::toString() const {
	return "uctv";
}
