/*
 * UCTVInf.cpp
 *
 *  Created on: 8 avr. 2010
 *      Author: jnschaeffer
 *
 * Copyright 2010 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "UCTVInf.h"
#include <math.h>
#include <time.h>
#include <cfloat>

UCTVInf::UCTVInf(double effectiveness, double propogation, double fpu, int k, double beta, bool useAIR):
	UCTV(effectiveness,propogation, fpu),
	numArms(k),
	beta(beta < 1 ? beta / 2.0 : beta / (1.0 + beta)),
	useAIR(useAIR)
{	}

action_t UCTVInf::select(GameState& root) {
	seed_t& seed = root.searchData.customSeed;
	action_t& arms = root.searchData.customAction;

	// If we haven't visited this node yet, initialize it with a seed and establish K new
	// arms.
	if(seed == 0) {
		seed = mGenerator.nextSeed();
		arms = numArms;
	}

	// Seed the random number generator so we can get the best move so far.
	RandomGenerator armGenerator(seed);

	if(useAIR == true) {
		if(arms < pow((double)root.searchData.numVisits,beta)) {
			if(arms < root.countChildren()) arms++;
		}
	}

	double bestVal = -DBL_MAX;
	double curVal = bestVal;
	action_t bestAction = 0;

	for(unsigned int i = 0; i < arms; i++) {
		action_t childIdx = armGenerator.nextAction(root.countChildren());
		//SearchData data = root.getChildSearchData(childIdx);
		curVal = evaluate(root.getChild(childIdx));
		if(curVal >= bestVal) {
			bestAction = childIdx;
			bestVal = curVal;
			//if(bestVal == DBL_MAX) break;
		}
	}

	return bestAction;
}

UCTVInf::~UCTVInf()
{ }

std::string UCTVInf::toString() const {
	std::stringstream out;
	out << "uctvinf-" << numArms;
	if(useAIR) out << "-air";
	return out.str();
}
