/*
 * GameState.cpp
 *
 *  Created on: Mar 19, 2010
 *      Author: dbjorge
 *
 * Copyright 2010 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <vector>
#include "GameState.h"
#include "GameTreeGenerator.h"


GameState::~GameState()
{ }

GameState::GameState(GameTreeGenerator& generator, action_t numChildren) :
		mGenerator(generator),
		mNumChildren(numChildren),
		mDepth(0U),
		mParent(NULL),
		mChildren((GameState**) generator.nextChildArray()),
		mNextUnvisitedChild(0),
		searchData()
{  }

GameState::GameState(GameTreeGenerator& generator, GameState& parent, action_t numChildren) :
		mGenerator(generator),
		mNumChildren(numChildren),
		mDepth(parent.getDepth()+1),
		mParent(&parent),
		mChildren((GameState**) generator.nextChildArray()),
		mNextUnvisitedChild(0),
		searchData()
{  }


action_t GameState::countVisitedChildren() {
	return mNextUnvisitedChild;
}

action_t GameState::getNextUnvisitedAction() {
	return mNextUnvisitedChild;
}

GameState& GameState::getVisitedChild(action_t index) {
	GameState* child = mChildren[index];
	if(child == NULL) {
		return *(mChildren[index] = generateChild(index));
	} else {
		return *child;
	}
}

GameState& GameState::getChild(action_t index) {
	GameState* child = mChildren[index];
	if(child == NULL) {
		if(index >= mNextUnvisitedChild) mNextUnvisitedChild = index + 1;
		return *(mChildren[index] = generateChild(index));
	} else {
		return *child;
	}
}

void GameState::resetChildArray() {
	mChildren = (GameState**) mGenerator.nextChildArray();
}

static const SearchData blankSearchData = SearchData();

SearchData GameState::getChildSearchData(action_t index) {
	GameState* child = mChildren[index];
	if(child == NULL) {
		return blankSearchData;
	} else {
		return child->searchData;
	}
}
