/*
 * SearchData.h
 *
 *  Created on: Mar 24, 2010
 *      Author: dbjorge
 *
 * Copyright 2010 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef SEARCHDATA_H_
#define SEARCHDATA_H_

#include "../gametree/Action.h"
#include "../gametree/RandomGenerator.h"

/**
 * Provides a collection of data storage variables for search algorithms
 * to use as they please. None of the base framework will ever involve itself
 * with these values - they are purely for the algorithms to use as they see
 * fit. They are defined here instead of being specific to the algorithms
 * (with a void* or something) because being able to allocate them statically
 * within states is significantly faster than needing to allocate them separately
 * from the nodes.
 */
struct SearchData {
	unsigned long numVisits;
	double empiricalValue;
	double estimatedValue;

	unsigned int customUInt;
	int customInt;
	double customDouble;
	player_t customPlayer;
	action_t customAction;
	seed_t   customSeed;


	SearchData() :
		numVisits(0),
		empiricalValue(0.5),
		estimatedValue(0.5),

		customUInt(0),
		customInt(0),
		customDouble(0.0),
		customPlayer(NEITHER),
		customAction(0),
		customSeed(0)
	{}
};

#endif /* SEARCHDATA_H_ */
