/*
 * SearcherFactory.cpp
 *
 *  Created on: Mar 19, 2010
 *      Author: dbjorge
 *
 * Copyright 2010 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "SearcherFactory.h"
#include "Searcher.h"
#include "SearchAlgorithm.h"
#include "../algorithms/AllAlgorithms.h"
#include <vector>
#include <list>

using namespace std;

SearcherFactory::SearcherFactory()
{ }

vector<Searcher>& SearcherFactory::generateSearchers(
		vector<string>& names, unsigned int max, double effectiveness) {

	vector<Searcher>* sList = new vector<Searcher>();
	sList->reserve(names.size());

	vector<string>::iterator ni;

	for(ni = names.begin(); ni != names.end(); ni++) {
		Searcher tmp(*ni, generateAlgorithm(*ni, effectiveness),max);
		sList->push_back(tmp);
	}

	return *sList;
}

bool isNum(char c) {
	return (c == '-') || (c == '.') || (c >= '0' && c <= '9');
}

SearchAlgorithm& SearcherFactory::generateAlgorithm(string name, double effectiveness) {
	string baseName;
	vector<double> args;
	int startArgIndex = -1;

	//First find base name
	for(unsigned int i=0 ; i<name.size() ;i++) {
		if(isNum(name[i])) {
			baseName = name.substr(0,i);
			startArgIndex = i;
			break;
		}
	}
	if(startArgIndex == -1) {
		baseName = name;
	}

	//Now parse any args
	if(startArgIndex != -1) {
		for(unsigned int i=startArgIndex+1;i<name.size();i++) {
			if(!isNum(name[i])) {
				if(i != (unsigned int) startArgIndex) {
					int size = i - startArgIndex;
					args.push_back(atof(name.substr(startArgIndex, size).c_str()));
				}
				startArgIndex = i+1;
			}
		}
		int remainingSize = name.size() - startArgIndex;
		if(remainingSize > 0) {
			args.push_back(atof(name.substr(startArgIndex, remainingSize).c_str()));
		}
	}

	SearchAlgorithm *newAlg = algreg::generateRegisteredAlgorithm(baseName, args);

	if(newAlg == NULL) {
		std::cout << "Warning: Could not parse algorithm name " << name << ", using random" << std::endl;
		newAlg = new RandomSearchAlg();
	}

	allocatedAlgs.push_back(newAlg);
	return *newAlg;

}

SearcherFactory::~SearcherFactory() {

	list<SearchAlgorithm*>::iterator ai;

	for(ai = allocatedAlgs.begin(); ai != allocatedAlgs.end(); ai++) {
		delete *ai;
	}
}
